/*
 * 쐬: 2005/01/10
 *
 */
package jp.starlogic.makotan.jooreport.impl;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import jp.starlogic.makotan.Exception.FileNotFoundRuntimeException;
import jp.starlogic.makotan.Exception.TemplateRuntimeException;
import jp.starlogic.makotan.buridicon.util.ScriptProcessor;
import jp.starlogic.makotan.jooreport.JooDocumentConverter;
import jp.starlogic.makotan.jooreport.JooTemplate;

import org.seasar.framework.exception.IORuntimeException;

import com.wonderlogic.jootemplates.Template;
import com.wonderlogic.jootemplates.TemplateException;

/**
 * @author makotan
 *
 */
public class JooTemplateImpl implements JooTemplate {
    private String templeteSXWName = null;
    private boolean imageSupport = false;
    private String outputOGNLName = null;
    private ThreadLocal localTemplate = new ThreadLocal();
    private JooDocumentConverter converter;

    public void setImage(String name, String fileName) {
        Template tmplate = getTemplate();
        InputStream stream = getInputStream(fileName);
        try {
            tmplate.setImage(name,stream);
        } catch (TemplateException e) {
            throw new TemplateRuntimeException(e);
        }
    }
    
    public void setImage(String name, BufferedImage image) {
        Template tmplate = getTemplate();
        try {
            tmplate.setImage(name,image);
        } catch (TemplateException e) {
            throw new TemplateRuntimeException(e);
        }
    }
    
    protected File createTempFile(String prefix, String suffix) {
        File tmpFile;
        try {
            tmpFile = File.createTempFile(prefix,suffix);
        } catch (IOException e) {
            throw new IORuntimeException(e);
        }
        return tmpFile;
    }
    
    public File createDocument(Object model) {
        File tmpSxwFile = createTempFile("joo","tmpl");
        FileOutputStream outStream = getFileOutputStream(tmpSxwFile);
        Template tmplate = getTemplate();
        try {
            tmplate.createDocument(model,outStream);
        } catch (IOException e) {
            throw new IORuntimeException(e);
        } catch (TemplateException e) {
            throw new TemplateRuntimeException(e);
        }
        
        File pdfFile = getPdfFile(model);
        getConverter().convert(tmpSxwFile,pdfFile);
        tmpSxwFile.delete();
        return pdfFile;
    }
    
    protected File getPdfFile(Object model) {
        File pdfFile;
        if(outputOGNLName==null) {
            pdfFile = createTempFile("joo","pdf");
        }else{
            ScriptProcessor script = new ScriptProcessor();
            String pdfFileName = (String)script.getValue(outputOGNLName,model);
            pdfFile = new File(pdfFileName);
        }
        return pdfFile;
    }
    
    protected FileOutputStream getFileOutputStream(File outFile) {
        FileOutputStream outStream;
        try {
            outStream = new FileOutputStream(outFile);
        } catch (FileNotFoundException e) {
            throw new FileNotFoundRuntimeException(e);
        }
        return outStream;
    }
    
    protected InputStream getInputStream(String fileName) {
        InputStream stream;
        try {
            stream = new FileInputStream(templeteSXWName);
        } catch (FileNotFoundException e) {
            throw new FileNotFoundRuntimeException(e);
        }
        return stream;
    }
    
    protected Template getTemplate() {
        Template tmplate = (Template)localTemplate.get();
        if(tmplate==null) {
            InputStream stream = getInputStream(templeteSXWName);
            try {
                tmplate = new Template(stream,imageSupport);
            } catch (IOException e1) {
                throw new IORuntimeException(e1);
            } catch (TemplateException e1) {
                throw new TemplateRuntimeException(e1);
            }
            localTemplate.set(tmplate);
        }
        return tmplate;
    }
    

    public boolean isImageSupport() {
        return imageSupport;
    }
    public void setImageSupport(boolean imageSupport) {
        this.imageSupport = imageSupport;
    }
    public String getOutputOGNLName() {
        return outputOGNLName;
    }
    public void setOutputOGNLName(String outputOGNLName) {
        this.outputOGNLName = outputOGNLName;
    }
    public String getTempleteSXWName() {
        return templeteSXWName;
    }
    public void setTempleteSXWName(String templeteSXWName) {
        this.templeteSXWName = templeteSXWName;
    }
    public JooDocumentConverter getConverter() {
        return converter;
    }
    public void setConverter(JooDocumentConverter converter) {
        this.converter = converter;
    }
}
